package edu.nyu.scps.animate;

import android.animation.ObjectAnimator;
import android.animation.TypeEvaluator;
import android.animation.ValueAnimator;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.PointF;
import android.view.MotionEvent;
import android.view.View;
import android.view.animation.LinearInterpolator;


public class AnimateView extends View {
    private PointF center = new PointF();	//of circle
    private float radius;
    private Paint paint = new Paint(Paint.ANTI_ALIAS_FLAG);

    public AnimateView(Context context) {
        super(context);
        setBackgroundColor(Color.WHITE);   //of entire AnimateView
        paint.setColor(Color.RED);
        paint.setStyle(Paint.Style.FILL);

        setOnTouchListener(new OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent motionEvent) {
                switch (motionEvent.getAction()) {
                    case MotionEvent.ACTION_DOWN:

                        ObjectAnimator objectAnimator = ObjectAnimator.ofObject(
                                AnimateView.this,
                                "center",   //name of field
                                new PointFEvaluator(),
                                new PointF(motionEvent.getX(), motionEvent.getY())
                        );

                        //Call the AnimateView's onDraw whenever the ObjectAnimator
                        //updates the value of the AnimateView's center field.

                        objectAnimator.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
                            @Override
                            public void onAnimationUpdate(ValueAnimator valueAnimator) {
                                //At this point, the animation has been running for
                                //valueAnimator.getCurrentPlayTime() milliseconds.
                                invalidate();   //trigger a call to onDraw
                            }
                        });

                        objectAnimator.setInterpolator(new LinearInterpolator());
                        objectAnimator.setDuration(3000L); //milliseconds, defaults to 300
                        objectAnimator.start();
                        return true;

                    default:
                        return false;
                }
            }
        });
    }

    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        super.onLayout(changed, left, top, right, bottom);
        center.set(getWidth() / 2f, getHeight() / 2f);
        radius = .1f * Math.min(getWidth(), getHeight());
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        canvas.drawCircle(center.x, center.y, radius, paint);
    }

    //The ObjectAnimator calls this getter.

    public PointF getCenter() {
        return center;
    }

    //The ObjectAnimator calls this setter.

    public void setCenter(PointF newCenter) {
        center.set(newCenter);
    }
}

/*
The ObjectAnimator calls the method of this class.
If fraction == 0f, evaluate returns startValue.
If fraction == 1f, evaluate returns endValue.
If fraction is in between, evaluate returns a PointF between startValue and endValue.
<Angle brackets> for Java "generic".
*/

class PointFEvaluator implements TypeEvaluator<PointF> {
    @Override
    public PointF evaluate(float fraction, PointF startValue, PointF endValue) {
        float x = (1f - fraction) * startValue.x + fraction * endValue.x;
        float y = (1f - fraction) * startValue.y + fraction * endValue.y;
        return new PointF(x, y);
    }
}
